#   ************    LibreSilicon's StdCellLibrary   *******************
#
#   Organisation:   Chipforge
#                   Germany / European Union
#
#   Profile:        Chipforge focus on fine System-on-Chip Cores in
#                   Verilog HDL Code which are easy understandable and
#                   adjustable. For further information see
#                           www.chipforge.org
#                   there are projects from small cores up to PCBs, too.
#
#   File:           StdCellLib/Catalog/stacked4_cells.mk
#
#   Purpose:        Makefile for Cell Generation with popcorn
#
#   ************    GNU Make 3.80 Source Code       ****************
#
#   ////////////////////////////////////////////////////////////////
#
#   Copyright (c)   2018, 2019 by
#                   chipforge - <popcorn@nospam.chipforge.org>
#   All rights reserved.
#
#       This Standard Cell Library is licensed under the Libre Silicon
#       public license; you can redistribute it and/or modify it under
#       the terms of the Libre Silicon public license as published by
#       the Libre Silicon alliance, either version 1 of the License, or
#       (at your option) any later version.
#
#       This design is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#       See the Libre Silicon Public License for more details.
#
#   ////////////////////////////////////////////////////////////////////

#   ----------------------------------------------------------------
#               DESCRIPTION
#   ----------------------------------------------------------------

#   list all dependencies for cells with 4 stacked transistors
#
include stacked3_cells.mk

#   ----------------------------------------------------------------
#               CELL TARGETS
#   ----------------------------------------------------------------

#   --------    already buffered    --------------------------------

ifdef BUFFERED

CELLS +=        AND4.cell \
                AO23.cell \
                AO41.cell \
                AOA212.cell \
                AOA311.cell \
                OA22.cell \
                OA23.cell \
                OA3111.cell \
                OA41.cell \
                OA4111.cell \
                OAO2111.cell \
                OAO311.cell \
                OAOA2211.cell \
                OAOA3211.cell \
                OR4.cell

AND4.cell:           DESCR = "4-input AND gate"
AND4.cell:           AND3.cell
	$(POPCORN) -m nand -c $@ $< > $@

AO23.cell:           DESCR = "2-3-input AND-OR gate"
AO23.cell:           OR4.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AO41.cell:           DESCR = "4-1-input AND-OR gate"
AO41.cell:           AO31.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AOA212.cell:         DESCR = "2-1-2-input AND-OR-AND gate"
AOA212.cell:         OA22.cell
	$(POPCORN) -m nand -c $@ $< > $@

AOA311.cell:         DESCR = "3-1-1-input AND-OR-AND gate"
AOA311.cell:         AOA211.cell
	$(POPCORN) -m aoi -c $@ $< > $@

OA22.cell:           DESCR = "2-2-input OR-AND gate"
OA22.cell:           AND3.cell
	$(POPCORN) -m oai -c $@ $< > $@

OA23.cell:           DESCR = "2-3-input OR-AND gate"
OA23.cell:           AND4.cell
	$(POPCORN) -m oai -c $@ $< > $@

OA33.cell:           DESCR = "3-3-input OR-AND gate"
OA33.cell:           OA23.cell
	$(POPCORN) -m oai -c $@ $< > $@

OA41.cell:           DESCR = "4-1-input OR-AND gate"
OA41.cell:           OA31.cell
	$(POPCORN) -m oai -c $@ $< > $@

OA43.cell:           DESCR = "4-3-input OR-AND gate"
OA43.cell:           OA33.cell
	$(POPCORN) -m oai -c $@ $< > $@

OAO311.cell:         DESCR = "3-1-1-input OR-AND-OR gate"
OAO311.cell:         OAO211.cell
	$(POPCORN) -m oai -c $@ $< > $@

OAOA2211.cell:       DESCR = "2-2-1-1-input OR-AND-OR-AND gate"
OAOA2211.cell:       AOA311.cell
	$(POPCORN) -m nor -c $@ $< > $@

OAOA3211.cell:       DESCR = "3-2-1-1-input OR-AND-OR-AND gate"
OAOA3211.cell:       OAOA2211.cell
	$(POPCORN) -m oai -c $@ $< > $@

OR4.cell:            DESCR = "4-input OR gate"
OR4.cell:            OR3.cell
	$(POPCORN) -m nor -c $@ $< > $@

#   --------    now buffered    ------------------------------------

else
ifeq ($(BUFFER),4)

CELLS +=        AND4.cell \
                AO23.cell \
                AO41.cell \
                AOA212.cell \
                AOA311.cell \
                OA22.cell \
                OA23.cell \
                OA3111.cell \
                OA41.cell \
                OA4111.cell \
                OAO2111.cell \
                OAO311.cell \
                OAOA2211.cell \
                OAOA3211.cell \
                OR4.cell

AND4.cell:           DESCR = "4-input AND gate"
AND4.cell:           NAND3.cell
	$(POPCORN) -m nand -c $@ $< > $@

AO23.cell:           DESCR = "2-3-input AND-OR gate"
AO23.cell:           OR4.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AO41.cell:           DESCR = "4-1-input AND-OR gate"
AO41.cell:           AOI31.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AOA212.cell:         DESCR = "2-1-2-input AND-OR-AND gate"
AOA212.cell:         OAI22.cell
	$(POPCORN) -m nand -c $@ $< > $@

AOA311.cell:         DESCR = "3-1-1-input AND-OR-AND gate"
AOA311.cell:         AOAI211.cell
	$(POPCORN) -m aoi -c $@ $< > $@

OA22.cell:           DESCR = "2-2-input OR-AND gate"
OA22.cell:           AND3.cell
	$(POPCORN) -m oai -c $@ $< > $@

OA23.cell:           DESCR = "2-3-input OR-AND gate"
OA23.cell:           AND4.cell
	$(POPCORN) -m oai -c $@ $< > $@

OA33.cell:           DESCR = "3-3-input OR-AND gate"
OA33.cell:           OA23.cell
	$(POPCORN) -m oai -c $@ $< > $@

OA41.cell:           DESCR = "4-1-input OR-AND gate"
OA41.cell:           OAI31.cell
	$(POPCORN) -m oai -c $@ $< > $@

OA43.cell:           DESCR = "4-3-input OR-AND gate"
OA43.cell:           OA33.cell
	$(POPCORN) -m oai -c $@ $< > $@

OAO311.cell:         DESCR = "3-1-1-input OR-AND-OR gate"
OAO311.cell:         OAOI211.cell
	$(POPCORN) -m oai -c $@ $< > $@

OAOA2211.cell:       DESCR = "2-2-1-1-input OR-AND-OR-AND gate"
OAOA2211.cell:       AOA311.cell
	$(POPCORN) -m nor -c $@ $< > $@

OAOA3211.cell:       DESCR = "3-2-1-1-input OR-AND-OR-AND gate"
OAOA3211.cell:       OAOA2211.cell
	$(POPCORN) -m oai -c $@ $< > $@

OR4.cell:            DESCR = "4-input OR gate"
OR4.cell:            NOR3.cell
	$(POPCORN) -m nor -c $@ $< > $@

BUFFERED = true

#   --------    not buffered    ------------------------------------

else

CELLS +=        AOAI212.cell \
                AOAI311.cell \
                AOI23.cell \
                AOI41.cell \
                NAND4.cell \
                NOR4.cell \
                OAI23.cell \
                OAI3111.cell \
                OAI41.cell \
                OAI4111.cell \
                OAOAI2211.cell \
                OAOAI3211.cell \
                OAOI311.cell

AOAI212.cell:        DESCR = "2-1-2-input AND-OR-AND-Invert gate"
AOAI212.cell:        OAI22.cell
	$(POPCORN) -m nand -c $@ $< > $@

AOAI311.cell:        DESCR = "3-1-1-input AND-OR-AND-Invert gate"
AOAI311.cell:        AOAI211.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AOI23.cell:          DESCR = "2-3-input AND-OR-Invert gate"
AOI23.cell:          NOR4.cell
	$(POPCORN) -m aoi -c $@ $< > $@

AOI41.cell:          DESCR = "4-1-input AND-OR-Invert gate"
AOI41.cell:          AOI31.cell
	$(POPCORN) -m aoi -c $@ $< > $@

NAND4.cell:          DESCR = "4-input Not-AND (or NAND) gate"
NAND4.cell:          NAND3.cell
	$(POPCORN) -m nand -c $@ $< > $@

NOR4.cell:           DESCR = "4-input Not-OR (or NOR) gate"
NOR4.cell:           NOR3.cell
	$(POPCORN) -m nor -c $@ $< > $@

OAI23.cell:          DESCR = "2-3-input OR-AND-Invert gate"
OAI23.cell:          NAND4.cell
	$(POPCORN) -m oai -c $@ $< > $@

OAI41.cell:          DESCR = "4-1-input OR-AND-Invert gate"
OAI41.cell:          OAI31.cell
	$(POPCORN) -m oai -c $@ $< > $@

OAI33.cell:          DESCR = "3-1-input OR-AND-Invert gate"
OAI33.cell:          OAI23.cell
	$(POPCORN) -m oai -c $@ $< > $@

OAI43.cell:          DESCR = "4-3-input OR-AND-Invert gate"
OAI43.cell:          OAI33.cell
	$(POPCORN) -m oai -c $@ $< > $@

OAOAI2211.cell:      DESCR = "2-2-1-1-input OR-AND-OR-AND-Invert gate"
OAOAI2211.cell:      AOAI311.cell
	$(POPCORN) -m nor -c $@ $< > $@

OAOAI3211.cell:      DESCR = "3-2-1-1-input OR-AND-OR-AND-Invert gate"
OAOAI3211.cell:      OAOAI2211.cell
	$(POPCORN) -m oai -c $@ $< > $@

OAOI311.cell:        DESCR = "3-1-1-input OR-AND-OR-Invert gate"
OAOI311.cell:        OAOI211.cell
	$(POPCORN) -m oai -c $@ $< > $@

endif
endif
